declare type StateType = 'Task' | 'Map' | 'Pass' | 'Wait' | 'Choice' | 'Succeed' | 'Fail';

interface BaseState {
    Type: StateType;
    Comment?: string;
}

declare type PayloadTemplate = object;
interface CanHaveInputPath {
    InputPath?: string | null;
}
interface CanHaveParameters {
    Parameters?: PayloadTemplate;
}
interface CanHaveResultSelector {
    ResultSelector?: PayloadTemplate;
}
interface CanHaveResultPath {
    ResultPath?: string | null;
}
interface CanHaveOutputPath {
    OutputPath?: string | null;
}

declare type StringOperatorNames = 'StringEquals' | 'StringLessThan' | 'StringGreaterThan' | 'StringLessThanEquals' | 'StringGreaterThanEquals' | 'StringMatches';
declare type StringPathOperatorNames = 'StringEqualsPath' | 'StringLessThanPath' | 'StringGreaterThanPath' | 'StringLessThanEqualsPath' | 'StringGreaterThanEqualsPath';
declare type NumericOperatorNames = 'NumericEquals' | 'NumericLessThan' | 'NumericGreaterThan' | 'NumericLessThanEquals' | 'NumericGreaterThanEquals';
declare type NumericPathOperatorNames = 'NumericEqualsPath' | 'NumericLessThanPath' | 'NumericGreaterThanPath' | 'NumericLessThanEqualsPath' | 'NumericGreaterThanEqualsPath';
declare type BooleanOperatorNames = 'BooleanEquals';
declare type BooleanPathOperatorNames = 'BooleanEqualsPath';
declare type TimestampOperatorNames = 'TimestampEquals' | 'TimestampLessThan' | 'TimestampGreaterThan' | 'TimestampLessThanEquals' | 'TimestampGreaterThanEquals';
declare type TimestampPathOperatorNames = 'TimestampEqualsPath' | 'TimestampLessThanPath' | 'TimestampGreaterThanPath' | 'TimestampLessThanEqualsPath' | 'TimestampGreaterThanEqualsPath';
declare type TypeTestOperatorNames = 'IsNull' | 'IsPresent' | 'IsNumeric' | 'IsString' | 'IsBoolean' | 'IsTimestamp';
declare type StringComparisonOperator = {
    [P in StringOperatorNames]?: string;
};
declare type StringPathComparisonOperator = {
    [P in StringPathOperatorNames]?: string;
};
declare type NumericComparisonOperator = {
    [P in NumericOperatorNames]?: number;
};
declare type NumericPathComparisonOperator = {
    [P in NumericPathOperatorNames]?: string;
};
declare type BooleanComparisonOperator = {
    [P in BooleanOperatorNames]?: boolean;
};
declare type BooleanPathComparisonOperator = {
    [P in BooleanPathOperatorNames]?: string;
};
declare type TimestampComparisonOperator = {
    [P in TimestampOperatorNames]?: string;
};
declare type TimestampPathComparisonOperator = {
    [P in TimestampPathOperatorNames]?: string;
};
declare type TypeTestComparisonOperator = {
    [P in TypeTestOperatorNames]?: true;
};
declare type ComparisonOperator = StringComparisonOperator | StringPathComparisonOperator | NumericComparisonOperator | NumericPathComparisonOperator | BooleanComparisonOperator | BooleanPathComparisonOperator | TimestampComparisonOperator | TimestampPathComparisonOperator | TypeTestComparisonOperator;
declare type BaseDataTestExpression = {
    Variable: string;
};
declare type DataTestExpression = BaseDataTestExpression & ComparisonOperator;
declare type BooleanExpression = {
    And?: ChoiceRuleWithoutNext[];
    Or?: ChoiceRuleWithoutNext[];
    Not?: ChoiceRuleWithoutNext;
};
declare type ChoiceRuleWithoutNext = DataTestExpression | BooleanExpression;
declare type ChoiceRule = (DataTestExpression | BooleanExpression) & {
    Next: string;
};
interface BaseChoiceState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Choice';
    Choices: ChoiceRule[];
    Default?: string;
}
declare type ChoiceState = BaseChoiceState;

interface EndableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    End: true;
}
interface SucceedOrFailState extends BaseState {
    Type: Extract<StateType, 'Succeed' | 'Fail'>;
}
declare type TerminalState = EndableState | SucceedOrFailState;

interface BaseFailState extends BaseState {
    Type: 'Fail';
    Cause?: string;
    Error?: string;
}
declare type FailState = TerminalState & BaseFailState;

interface NextableState extends BaseState {
    Type: Exclude<StateType, 'Choice' | 'Succeed' | 'Fail'>;
    Next: string;
}
declare type IntermediateState = NextableState;

interface BaseMapState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Map';
    Iterator: Omit<StateMachineDefinition, 'Version' | 'TimeoutSeconds'>;
    ItemsPath?: string;
    MaxConcurrency?: number;
}
declare type MapState = (IntermediateState | TerminalState) & BaseMapState;

declare type JSONValue = null | boolean | number | string | object | any[];

interface BasePassState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Pass';
    Result?: JSONValue;
}
declare type PassState = (IntermediateState | TerminalState) & BasePassState;

interface BaseSucceedState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Succeed';
}
declare type SucceedState = TerminalState & BaseSucceedState;

interface BaseTaskState extends BaseState, CanHaveInputPath, CanHaveParameters, CanHaveResultSelector, CanHaveResultPath, CanHaveOutputPath {
    Type: 'Task';
    Resource: string;
}
declare type TaskState = (IntermediateState | TerminalState) & BaseTaskState;

interface BaseWaitState extends BaseState, CanHaveInputPath, CanHaveOutputPath {
    Type: 'Wait';
    Seconds?: number;
    Timestamp?: string;
    SecondsPath?: string;
    TimestampPath?: string;
}
declare type WaitState = (IntermediateState | TerminalState) & BaseWaitState;

declare type AllStates = TaskState | MapState | PassState | WaitState | ChoiceState | SucceedState | FailState;

interface StateMachineDefinition {
    States: Record<string, AllStates>;
    StartAt: string;
    Comment?: string;
    Version?: string;
    TimeoutSeconds?: number;
}

declare type TaskStateResourceLocalHandler = {
    [taskStateName: string]: (...args: any) => any;
};
declare type WaitStateTimeOverride = {
    [waitStateName: string]: number;
};
interface Overrides {
    taskResourceLocalHandlers?: TaskStateResourceLocalHandler;
    waitTimeOverrides?: WaitStateTimeOverride;
}
interface RunOptions {
    overrides?: Overrides;
}
interface ValidationOptions {
    readonly checkPaths?: boolean;
    readonly checkArn?: boolean;
}

declare class StateMachine {
    /**
     * The name of the state currently being executed.
     */
    private currStateName;
    /**
     * The current state being executed.
     */
    private currState;
    /**
     * The unmodified input to the current state.
     */
    private rawInput;
    /**
     * The input that can be modified according to the `InputPath` and `Parameters` fields of the current state.
     */
    private currInput;
    /**
     * The result that can be modified according to the `ResultSelector`, `ResultPath` and `OutputPath` fields of the current state.
     */
    private currResult;
    /**
     * The context object of the state machine.
     */
    private context;
    /**
     * A map of all states defined in the state machine.
     */
    private readonly states;
    /**
     * A map of functions to handle each type of state.
     */
    private readonly stateHandlers;
    /**
     * Options to control whether to apply certain validations to the state machine definition.
     */
    private readonly validationOptions;
    /**
     * Constructs a new state machine.
     * @param definition The state machine definition defined using the Amazon States Language (https://states-language.net/spec.html).
     * @param validationOptions Options to control whether to apply certain validations to the definition.
     * These options also apply to state machines defined in  the `Iterator` field of `Map` states.
     */
    constructor(definition: StateMachineDefinition, validationOptions?: ValidationOptions);
    /**
     * Executes the state machine, running through the states specified in the definiton.
     * @param input The input to pass to this state machine execution.
     * @param options Miscellaneous options to control certain behaviors of the execution.
     */
    run(input: JSONValue, options?: RunOptions): Promise<JSONValue>;
    /**
     * Process the current input according to the path defined in the `InputPath` field, if specified in the current state.
     * @returns
     * * If `InputPath` is not specified, returns the current input unmodified.
     * * If `InputPath` is `null`, returns an empty object (`{}`).
     * * If `InputPath` is a string, it's considered a JSONPath and the selected portion of the current input is returned.
     */
    private processInputPath;
    /**
     * Recursively process a payload template to resolve the properties that are JSONPaths.
     * @param payloadTemplate The payload template to process.
     * @param json The object to evaluate with JSONPath (whether of null, boolean, number, string, object, or array type).
     * @returns The processed payload template.
     */
    private processPayloadTemplate;
    /**
     * Process the current input according to the `InputPath` and `Parameters` fields.
     */
    private processInput;
    /**
     * Process the current result according to the path defined in the `ResultPath` field, if specified in the current state.
     * @returns
     * * If `ResultPath` is not specified, returns the current result unmodified.
     * * If `ResultPath` is `null`, returns the raw input (i.e. the input passed to current state).
     * * If `ResultPath` is a string, it's considered a JSONPath and returns a combination of the raw input with the current result,
     * by placing the current result in the specified path.
     */
    private processResultPath;
    /**
     * Process the current result according to the path defined in the `OutputPath` field, if specified in the current state.
     * @returns
     * * If `OutputPath` is not specified, returns the current result unmodified.
     * * If `OutputPath` is `null`, returns an empty object (`{}`).
     * * If `OutputPath` is a string, it's considered a JSONPath and the selected portion of the current result is returned.
     */
    private processOutputPath;
    /**
     * Process the current result according to the `ResultSelector`, `ResultPath` and `OutputPath` fields.
     */
    private processResult;
    /**
     * Handler for task states.
     *
     * Invokes the Lambda function specified in the `Resource` field
     * and sets the current result of the state machine to the value returned by the Lambda.
     */
    private handleTaskState;
    /**
     * Handler for map states.
     *
     * Iterates over the current input items or the items of an array specified
     * by the `ItemsPath` field, and then processes each item by passing it
     * as the input to the state machine specified in the `Iterator` field.
     */
    private handleMapState;
    /**
     * Handler for pass states.
     *
     * If the `Result` field is specified, copies `Result` into the current result.
     * Else, copies the current input into the current result.
     */
    private handlePassState;
    /**
     * Handler for wait states.
     *
     * Pauses the state machine execution for a certain amount of time
     * based on one of the `Seconds`, `Timestamp`, `SecondsPath` or `TimestampPath` fields.
     */
    private handleWaitState;
    /**
     * Handler for choice states.
     *
     * Evaluates each choice rule specified in the `Choices` field.
     *
     * If one of the rules matches, then the state machine transitions to the
     * state specified in the `Next` field for that choice rule.
     *
     * If no rule matches but the `Default` field is specified,
     * then the next state will be the state specified in said field.
     *
     * If no rule matches and the `Default` field is not specified, throws a
     * States.NoChoiceMatched error.
     */
    private handleChoiceState;
    /**
     * Handler for succeed states.
     *
     * Ends the state machine execution successfully.
     */
    private handleSucceedState;
    /**
     * Handler for fail states.
     *
     * Ends the state machine execution and marks it as a failure.
     */
    private handleFailState;
    /**
     * Queries for a property in an object using a JSONPath expression.
     * @param pathExpression The JSONPath expression to query for.
     * @param json The object to evaluate (whether of null, boolean, number, string, object, or array type).
     * @returns The value of the property that was queried for, if found. Otherwise returns `undefined`.
     */
    private jsonQuery;
}

export { StateMachine };
